@@ -32,10 +32,8 @@ struct Curl_easy;
 
 #include "urldata.h" /* for struct Curl_easy */
 #include "mime.h"
-#include "vtls/vtls.h"
 #include "sendf.h"
 #include "strdup.h"
-#include "rand.h"
 #include "bufref.h"
 #include "curlx/fopen.h"
 #include "curlx/warnless.h"
