@@ -77,9 +77,6 @@
 
 #include "curl_setup.h"
 
-#include <limits.h>
-
-#include <curl/curl.h>
 #include "curlx/warnless.h"
 #include "parsedate.h"
 #include "curlx/strparse.h"
@@ -107,17 +104,19 @@ static int parsedate(const char *date, time_t *output);
 #if !defined(CURL_DISABLE_PARSEDATE) || !defined(CURL_DISABLE_FTP) || \
   !defined(CURL_DISABLE_FILE) || defined(USE_GNUTLS)
 /* These names are also used by FTP and FILE code */
-const char * const Curl_wkday[] =
-{"Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"};
-const char * const Curl_month[]=
-{ "Jan", "Feb", "Mar", "Apr", "May", "Jun",
-  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+const char * const Curl_wkday[] = {
+  "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
+};
+const char * const Curl_month[] = {
+  "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+};
 #endif
 
 #ifndef CURL_DISABLE_PARSEDATE
-static const char * const weekday[] =
-{ "Monday", "Tuesday", "Wednesday", "Thursday",
-  "Friday", "Saturday", "Sunday" };
+static const char * const weekday[] = {
+  "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"
+};
 
 struct tzinfo {
   char name[5];
@@ -127,83 +126,83 @@ struct tzinfo {
 /* Here's a bunch of frequently used time zone names. These were supported
    by the old getdate parser. */
 #define tDAYZONE -60       /* offset for daylight savings time */
-static const struct tzinfo tz[]= {
-  {"GMT", 0},              /* Greenwich Mean */
-  {"UT",  0},              /* Universal Time */
-  {"UTC", 0},              /* Universal (Coordinated) */
-  {"WET", 0},              /* Western European */
-  {"BST", 0 tDAYZONE},     /* British Summer */
-  {"WAT", 60},             /* West Africa */
-  {"AST", 240},            /* Atlantic Standard */
-  {"ADT", 240 tDAYZONE},   /* Atlantic Daylight */
-  {"EST", 300},            /* Eastern Standard */
-  {"EDT", 300 tDAYZONE},   /* Eastern Daylight */
-  {"CST", 360},            /* Central Standard */
-  {"CDT", 360 tDAYZONE},   /* Central Daylight */
-  {"MST", 420},            /* Mountain Standard */
-  {"MDT", 420 tDAYZONE},   /* Mountain Daylight */
-  {"PST", 480},            /* Pacific Standard */
-  {"PDT", 480 tDAYZONE},   /* Pacific Daylight */
-  {"YST", 540},            /* Yukon Standard */
-  {"YDT", 540 tDAYZONE},   /* Yukon Daylight */
-  {"HST", 600},            /* Hawaii Standard */
-  {"HDT", 600 tDAYZONE},   /* Hawaii Daylight */
-  {"CAT", 600},            /* Central Alaska */
-  {"AHST", 600},           /* Alaska-Hawaii Standard */
-  {"NT",  660},            /* Nome */ /* spellchecker:disable-line */
-  {"IDLW", 720},           /* International Date Line West */
-  {"CET", -60},            /* Central European */
-  {"MET", -60},            /* Middle European */
-  {"MEWT", -60},           /* Middle European Winter */
-  {"MEST", -60 tDAYZONE},  /* Middle European Summer */
-  {"CEST", -60 tDAYZONE},  /* Central European Summer */
-  {"MESZ", -60 tDAYZONE},  /* Middle European Summer */
-  {"FWT", -60},            /* French Winter */
-  {"FST", -60 tDAYZONE},   /* French Summer */
-  {"EET", -120},           /* Eastern Europe, USSR Zone 1 */
-  {"WAST", -420}, /* spellchecker:disable-line */
-                           /* West Australian Standard */
-  {"WADT", -420 tDAYZONE}, /* West Australian Daylight */
-  {"CCT", -480},           /* China Coast, USSR Zone 7 */
-  {"JST", -540},           /* Japan Standard, USSR Zone 8 */
-  {"EAST", -600},          /* Eastern Australian Standard */
-  {"EADT", -600 tDAYZONE}, /* Eastern Australian Daylight */
-  {"GST", -600},           /* Guam Standard, USSR Zone 9 */
-  {"NZT", -720},           /* New Zealand */
-  {"NZST", -720},          /* New Zealand Standard */
-  {"NZDT", -720 tDAYZONE}, /* New Zealand Daylight */
-  {"IDLE", -720},          /* International Date Line East */
+static const struct tzinfo tz[] = {
+  { "GMT",     0 },          /* Greenwich Mean */
+  { "UT",      0 },          /* Universal Time */
+  { "UTC",     0 },          /* Universal (Coordinated) */
+  { "WET",     0 },          /* Western European */
+  { "BST",     0 tDAYZONE }, /* British Summer */
+  { "WAT",    60 },          /* West Africa */
+  { "AST",   240 },          /* Atlantic Standard */
+  { "ADT",   240 tDAYZONE }, /* Atlantic Daylight */
+  { "EST",   300 },          /* Eastern Standard */
+  { "EDT",   300 tDAYZONE }, /* Eastern Daylight */
+  { "CST",   360 },          /* Central Standard */
+  { "CDT",   360 tDAYZONE }, /* Central Daylight */
+  { "MST",   420 },          /* Mountain Standard */
+  { "MDT",   420 tDAYZONE }, /* Mountain Daylight */
+  { "PST",   480 },          /* Pacific Standard */
+  { "PDT",   480 tDAYZONE }, /* Pacific Daylight */
+  { "YST",   540 },          /* Yukon Standard */
+  { "YDT",   540 tDAYZONE }, /* Yukon Daylight */
+  { "HST",   600 },          /* Hawaii Standard */
+  { "HDT",   600 tDAYZONE }, /* Hawaii Daylight */
+  { "CAT",   600 },          /* Central Alaska */
+  { "AHST",  600 },          /* Alaska-Hawaii Standard */
+  { "NT",    660 },          /* Nome */ /* spellchecker:disable-line */
+  { "IDLW",  720 },          /* International Date Line West */
+  { "CET",   -60 },          /* Central European */
+  { "MET",   -60 },          /* Middle European */
+  { "MEWT",  -60 },          /* Middle European Winter */
+  { "MEST",  -60 tDAYZONE }, /* Middle European Summer */
+  { "CEST",  -60 tDAYZONE }, /* Central European Summer */
+  { "MESZ",  -60 tDAYZONE }, /* Middle European Summer */
+  { "FWT",   -60 },          /* French Winter */
+  { "FST",   -60 tDAYZONE }, /* French Summer */
+  { "EET",  -120 },          /* Eastern Europe, USSR Zone 1 */
+  { "WAST", -420 }, /* spellchecker:disable-line */
+                             /* West Australian Standard */
+  { "WADT", -420 tDAYZONE }, /* West Australian Daylight */
+  { "CCT",  -480 },          /* China Coast, USSR Zone 7 */
+  { "JST",  -540 },          /* Japan Standard, USSR Zone 8 */
+  { "EAST", -600 },          /* Eastern Australian Standard */
+  { "EADT", -600 tDAYZONE }, /* Eastern Australian Daylight */
+  { "GST",  -600 },          /* Guam Standard, USSR Zone 9 */
+  { "NZT",  -720 },          /* New Zealand */
+  { "NZST", -720 },          /* New Zealand Standard */
+  { "NZDT", -720 tDAYZONE }, /* New Zealand Daylight */
+  { "IDLE", -720 },          /* International Date Line East */
   /* Next up: Military timezone names. RFC822 allowed these, but (as noted in
      RFC 1123) had their signs wrong. Here we use the correct signs to match
      actual military usage.
    */
-  {"A",  1 * 60},         /* Alpha */
-  {"B",  2 * 60},         /* Bravo */
-  {"C",  3 * 60},         /* Charlie */
-  {"D",  4 * 60},         /* Delta */
-  {"E",  5 * 60},         /* Echo */
-  {"F",  6 * 60},         /* Foxtrot */
-  {"G",  7 * 60},         /* Golf */
-  {"H",  8 * 60},         /* Hotel */
-  {"I",  9 * 60},         /* India */
+  { "A",   1 * 60 },         /* Alpha */
+  { "B",   2 * 60 },         /* Bravo */
+  { "C",   3 * 60 },         /* Charlie */
+  { "D",   4 * 60 },         /* Delta */
+  { "E",   5 * 60 },         /* Echo */
+  { "F",   6 * 60 },         /* Foxtrot */
+  { "G",   7 * 60 },         /* Golf */
+  { "H",   8 * 60 },         /* Hotel */
+  { "I",   9 * 60 },         /* India */
   /* "J", Juliet is not used as a timezone, to indicate the observer's local
      time */
-  {"K", 10 * 60},         /* Kilo */
-  {"L", 11 * 60},         /* Lima */
-  {"M", 12 * 60},         /* Mike */
-  {"N",  -1 * 60},         /* November */
-  {"O",  -2 * 60},         /* Oscar */
-  {"P",  -3 * 60},         /* Papa */
-  {"Q",  -4 * 60},         /* Quebec */
-  {"R",  -5 * 60},         /* Romeo */
-  {"S",  -6 * 60},         /* Sierra */
-  {"T",  -7 * 60},         /* Tango */
-  {"U",  -8 * 60},         /* Uniform */
-  {"V",  -9 * 60},         /* Victor */
-  {"W", -10 * 60},         /* Whiskey */
-  {"X", -11 * 60},         /* X-ray */
-  {"Y", -12 * 60},         /* Yankee */
-  {"Z", 0},                /* Zulu, zero meridian, a.k.a. UTC */
+  { "K",  10 * 60 },         /* Kilo */
+  { "L",  11 * 60 },         /* Lima */
+  { "M",  12 * 60 },         /* Mike */
+  { "N",  -1 * 60 },         /* November */
+  { "O",  -2 * 60 },         /* Oscar */
+  { "P",  -3 * 60 },         /* Papa */
+  { "Q",  -4 * 60 },         /* Quebec */
+  { "R",  -5 * 60 },         /* Romeo */
+  { "S",  -6 * 60 },         /* Sierra */
+  { "T",  -7 * 60 },         /* Tango */
+  { "U",  -8 * 60 },         /* Uniform */
+  { "V",  -9 * 60 },         /* Victor */
+  { "W", -10 * 60 },         /* Whiskey */
+  { "X", -11 * 60 },         /* X-ray */
+  { "Y", -12 * 60 },         /* Yankee */
+  { "Z", 0 },                /* Zulu, zero meridian, a.k.a. UTC */
 };
 
 /* returns:
@@ -260,7 +259,7 @@ static int checktz(const char *check, size_t len)
     size_t ilen = strlen(what->name);
     if((ilen == len) &&
        curl_strnequal(check, what->name, len))
-      return what->offset*60;
+      return what->offset * 60;
     what++;
   }
   return -1;
@@ -286,12 +285,13 @@ enum assume {
 static time_t time2epoch(int sec, int min, int hour,
                          int mday, int mon, int year)
 {
-  static const int month_days_cumulative[12] =
-    { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
+  static const int month_days_cumulative[12] = {
+    0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
+  };
   int leap_days = year - (mon <= 1);
   leap_days = ((leap_days / 4) - (leap_days / 100) + (leap_days / 400)
                - (1969 / 4) + (1969 / 100) - (1969 / 400));
-  return ((((time_t) (year - 1970) * 365
+  return ((((time_t)(year - 1970) * 365
             + leap_days + month_days_cumulative[mon] + mday - 1) * 24
            + hour) * 60 + min) * 60 + sec;
 }
@@ -304,13 +304,12 @@ static int oneortwodigit(const char *date, const char **endp)
   int num = date[0] - '0';
   if(ISDIGIT(date[1])) {
     *endp = &date[2];
-    return num*10 + (date[1] - '0');
+    return num * 10 + (date[1] - '0');
   }
   *endp = &date[1];
   return num;
 }
 
-
 /* HH:MM:SS or HH:MM and accept single-digits too */
 static bool match_time(const char *date,
                        int *h, int *m, int *s, char **endp)
@@ -444,11 +443,11 @@ static int parsedate(const char *date, time_t *output)
              anyone has a more authoritative source for the exact maximum time
              zone offsets, please speak up! */
           found = TRUE;
-          tzoff = (val/100 * 60 + val%100)*60;
+          tzoff = (val / 100 * 60 + val % 100) * 60;
 
           /* the + and - prefix indicates the local time compared to GMT,
              this we need their reversed math to get what we want */
-          tzoff = date[-1]=='+' ? -tzoff : tzoff;
+          tzoff = date[-1] == '+' ? -tzoff : tzoff;
         }
 
         else if((num_digits == 8) &&
@@ -457,9 +456,9 @@ static int parsedate(const char *date, time_t *output)
                 (mdaynum == -1)) {
           /* 8 digits, no year, month or day yet. This is YYYYMMDD */
           found = TRUE;
-          yearnum = val/10000;
-          monnum = (val%10000)/100-1; /* month is 0 - 11 */
-          mdaynum = val%100;
+          yearnum = val / 10000;
+          monnum = (val % 10000) / 100 - 1; /* month is 0 - 11 */
+          mdaynum = val % 100;
         }
 
         if(!found && (dignext == DATE_MDAY) && (mdaynum == -1)) {
@@ -596,27 +595,3 @@ int Curl_getdate_capped(const char *p, time_t *tp)
   int rc = parsedate(p, tp);
   return (rc == PARSEDATE_FAIL);
 }
-
-/*
- * Curl_gmtime() is a gmtime() replacement for portability. Do not use the
- * gmtime_r() or gmtime() functions anywhere else but here.
- *
- */
-
-CURLcode Curl_gmtime(time_t intime, struct tm *store)
-{
-  const struct tm *tm;
-#ifdef HAVE_GMTIME_R
-  /* thread-safe version */
-  tm = (struct tm *)gmtime_r(&intime, store);
-#else
-  /* !checksrc! disable BANNEDFUNC 1 */
-  tm = gmtime(&intime);
-  if(tm)
-    *store = *tm; /* copy the pointed struct to the local copy */
-#endif
-
-  if(!tm)
-    return CURLE_BAD_FUNCTION_ARGUMENT;
-  return CURLE_OK;
-}
